# Copyright (c) 2016, 2021, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Description of log analytics entity type.
  #
  class LogAnalytics::Models::LogAnalyticsEntityType
    CLOUD_TYPE_ENUM = [
      CLOUD_TYPE_CLOUD = 'CLOUD'.freeze,
      CLOUD_TYPE_NON_CLOUD = 'NON_CLOUD'.freeze,
      CLOUD_TYPE_ALL = 'ALL'.freeze,
      CLOUD_TYPE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    LIFECYCLE_STATE_ENUM = [
      LIFECYCLE_STATE_ACTIVE = 'ACTIVE'.freeze,
      LIFECYCLE_STATE_DELETED = 'DELETED'.freeze,
      LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    MANAGEMENT_AGENT_ELIGIBILITY_STATUS_ENUM = [
      MANAGEMENT_AGENT_ELIGIBILITY_STATUS_ELIGIBLE = 'ELIGIBLE'.freeze,
      MANAGEMENT_AGENT_ELIGIBILITY_STATUS_INELIGIBLE = 'INELIGIBLE'.freeze,
      MANAGEMENT_AGENT_ELIGIBILITY_STATUS_UNKNOWN = 'UNKNOWN'.freeze,
      MANAGEMENT_AGENT_ELIGIBILITY_STATUS_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** Log analytics entity type name.
    #
    # @return [String]
    attr_accessor :name

    # **[Required]** Internal name for the log analytics entity type.
    #
    # @return [String]
    attr_accessor :internal_name

    # Compartment Identifier [OCID] (https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm).
    # @return [String]
    attr_accessor :compartment_id

    # **[Required]** Log analytics entity type category. Category will be used for grouping and filtering.
    #
    # @return [String]
    attr_accessor :category

    # **[Required]** Log analytics entity type group. That can be CLOUD (OCI) or NON_CLOUD otherwise.
    #
    # @return [String]
    attr_reader :cloud_type

    # The parameters used in file patterns specified in log sources for this log analytics entity type.
    #
    # @return [Array<OCI::LogAnalytics::Models::EntityTypeProperty>]
    attr_accessor :properties

    # **[Required]** The current lifecycle state of the log analytics entity.
    #
    # @return [String]
    attr_reader :lifecycle_state

    # **[Required]** Time the log analytics entity type was created. An RFC3339 formatted datetime string.
    #
    # @return [DateTime]
    attr_accessor :time_created

    # **[Required]** Time the log analytics entity type was updated. An RFC3339 formatted datetime string.
    #
    # @return [DateTime]
    attr_accessor :time_updated

    # This field indicates whether logs for entities of this type can be collected using a management agent.
    #
    # @return [String]
    attr_reader :management_agent_eligibility_status

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'name': :'name',
        'internal_name': :'internalName',
        'compartment_id': :'compartmentId',
        'category': :'category',
        'cloud_type': :'cloudType',
        'properties': :'properties',
        'lifecycle_state': :'lifecycleState',
        'time_created': :'timeCreated',
        'time_updated': :'timeUpdated',
        'management_agent_eligibility_status': :'managementAgentEligibilityStatus'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'name': :'String',
        'internal_name': :'String',
        'compartment_id': :'String',
        'category': :'String',
        'cloud_type': :'String',
        'properties': :'Array<OCI::LogAnalytics::Models::EntityTypeProperty>',
        'lifecycle_state': :'String',
        'time_created': :'DateTime',
        'time_updated': :'DateTime',
        'management_agent_eligibility_status': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :name The value to assign to the {#name} property
    # @option attributes [String] :internal_name The value to assign to the {#internal_name} property
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [String] :category The value to assign to the {#category} property
    # @option attributes [String] :cloud_type The value to assign to the {#cloud_type} property
    # @option attributes [Array<OCI::LogAnalytics::Models::EntityTypeProperty>] :properties The value to assign to the {#properties} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    # @option attributes [DateTime] :time_created The value to assign to the {#time_created} property
    # @option attributes [DateTime] :time_updated The value to assign to the {#time_updated} property
    # @option attributes [String] :management_agent_eligibility_status The value to assign to the {#management_agent_eligibility_status} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.name = attributes[:'name'] if attributes[:'name']

      self.internal_name = attributes[:'internalName'] if attributes[:'internalName']

      raise 'You cannot provide both :internalName and :internal_name' if attributes.key?(:'internalName') && attributes.key?(:'internal_name')

      self.internal_name = attributes[:'internal_name'] if attributes[:'internal_name']

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.category = attributes[:'category'] if attributes[:'category']

      self.cloud_type = attributes[:'cloudType'] if attributes[:'cloudType']

      raise 'You cannot provide both :cloudType and :cloud_type' if attributes.key?(:'cloudType') && attributes.key?(:'cloud_type')

      self.cloud_type = attributes[:'cloud_type'] if attributes[:'cloud_type']

      self.properties = attributes[:'properties'] if attributes[:'properties']

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']

      self.time_created = attributes[:'timeCreated'] if attributes[:'timeCreated']

      raise 'You cannot provide both :timeCreated and :time_created' if attributes.key?(:'timeCreated') && attributes.key?(:'time_created')

      self.time_created = attributes[:'time_created'] if attributes[:'time_created']

      self.time_updated = attributes[:'timeUpdated'] if attributes[:'timeUpdated']

      raise 'You cannot provide both :timeUpdated and :time_updated' if attributes.key?(:'timeUpdated') && attributes.key?(:'time_updated')

      self.time_updated = attributes[:'time_updated'] if attributes[:'time_updated']

      self.management_agent_eligibility_status = attributes[:'managementAgentEligibilityStatus'] if attributes[:'managementAgentEligibilityStatus']

      raise 'You cannot provide both :managementAgentEligibilityStatus and :management_agent_eligibility_status' if attributes.key?(:'managementAgentEligibilityStatus') && attributes.key?(:'management_agent_eligibility_status')

      self.management_agent_eligibility_status = attributes[:'management_agent_eligibility_status'] if attributes[:'management_agent_eligibility_status']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] cloud_type Object to be assigned
    def cloud_type=(cloud_type)
      # rubocop:disable Style/ConditionalAssignment
      if cloud_type && !CLOUD_TYPE_ENUM.include?(cloud_type)
        OCI.logger.debug("Unknown value for 'cloud_type' [" + cloud_type + "]. Mapping to 'CLOUD_TYPE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @cloud_type = CLOUD_TYPE_UNKNOWN_ENUM_VALUE
      else
        @cloud_type = cloud_type
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lifecycle_state Object to be assigned
    def lifecycle_state=(lifecycle_state)
      # rubocop:disable Style/ConditionalAssignment
      if lifecycle_state && !LIFECYCLE_STATE_ENUM.include?(lifecycle_state)
        OCI.logger.debug("Unknown value for 'lifecycle_state' [" + lifecycle_state + "]. Mapping to 'LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @lifecycle_state = LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE
      else
        @lifecycle_state = lifecycle_state
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] management_agent_eligibility_status Object to be assigned
    def management_agent_eligibility_status=(management_agent_eligibility_status)
      # rubocop:disable Style/ConditionalAssignment
      if management_agent_eligibility_status && !MANAGEMENT_AGENT_ELIGIBILITY_STATUS_ENUM.include?(management_agent_eligibility_status)
        OCI.logger.debug("Unknown value for 'management_agent_eligibility_status' [" + management_agent_eligibility_status + "]. Mapping to 'MANAGEMENT_AGENT_ELIGIBILITY_STATUS_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @management_agent_eligibility_status = MANAGEMENT_AGENT_ELIGIBILITY_STATUS_UNKNOWN_ENUM_VALUE
      else
        @management_agent_eligibility_status = management_agent_eligibility_status
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        name == other.name &&
        internal_name == other.internal_name &&
        compartment_id == other.compartment_id &&
        category == other.category &&
        cloud_type == other.cloud_type &&
        properties == other.properties &&
        lifecycle_state == other.lifecycle_state &&
        time_created == other.time_created &&
        time_updated == other.time_updated &&
        management_agent_eligibility_status == other.management_agent_eligibility_status
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [name, internal_name, compartment_id, category, cloud_type, properties, lifecycle_state, time_created, time_updated, management_agent_eligibility_status].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
